      subroutine EMIC_Erese_Rela(xh0,thetad0,fpara,ioneta1,ioneta2,
     1 m,Eres_e,n2)
c     Relativistic regime
c     Calculating electron minimum resonant energy Eres_e in eV
c     omega - kz vz = m Omega_e / gamma
c     where gamma=(1-v**2/c**2)**(-1/2)
c     mass emas, charge -e
c     the wave mode is specified by n2mdl, defined in WavesWork.h
c     xh0: wave frequency normalized to H+ gyro
c     thetad0: wave normal angle in degree
c     fpara: wpe/wce 
c     ioneta1, ioneta2: H+, He+ composition
c     m: resonance harmonics number (m=-1)
c     Lunjin Chen
c     11/3/2009

      use mod_PhyConst
      use mod_WavesWork
	implicit none

	real*8 xh0,thetad0,fpara,ioneta1,ioneta2
      real*8 n2,n20,Eres_e,omegae,Estate0
      real*8 gamma0,gamma1
	real*8 DD33(3,3),dDDdkz,dDDdkp
      real*8 dDDdx
      integer*4 m
c      real*8 ww,kk
c      common/ERES_DATA/ ww,kk
      call ColdDispRelaSol(n20,xh0,thetad0,fpara,
     1 ioneta1,ioneta2,n2,DD33,dDDdx,dDDdkp,dDDdkz)
      Estate0=emas*clight**2/ev

      if (n2.gt.0) then 
            omegae=1.0d0/memp
            gamma1=omegae/(xh0*sqrt(n2)*cos(thetad0*rad))
            gamma0=sqrt(gamma1**2+1.0)
            Eres_e=(gamma0-1)*Estate0
      else
            Eres_e=-1e30
      endif
      end

      subroutine Res_Velocity_NonRela(xh0,thetad0,fpara,ioneta1,ioneta2,
     1 m,rmas,rq,vres,n2)
c     Nonrelativistic regime
c     Calculating resonant velocity for particle with
c     mass rmas*hmas, charge rq*e
c     the wave mode is specified by n2mdl, defined in WavesWork.h
c     xh0: wave frequency normalized to H+ gyro
c     thetad0: wave normal angle in degree
c     fpara: wpe/wce 
c     ioneta1, ioneta2: H+, He+ composition
c     m: resonance harmonics number
c     vres: resonant vz, which is calculated through
c     omega - kz vz = m Omega_s
c     Lunjin Chen
c     11/3/2009

      use mod_PhyConst
      use mod_WavesWork
	implicit none

	real*8 xh0,thetad0,fpara,ioneta1,ioneta2
      real*8 n2,n20,rmas,rq,vres,omegas
	real*8 DD33(3,3),dDDdkz,dDDdkp
      real*8 dDDdx
      integer*4 m
      call ColdDispRelaSol(n20,xh0,thetad0,fpara,
     1 ioneta1,ioneta2,n2,DD33,dDDdx,dDDdkp,dDDdkz)
      omegas=rq/rmas
      if (n2.gt.0) then 
            vres=clight/(sqrt(n2)*cos(thetad0*rad))*(1-m*omegas/xh0)
      else
            vres=-1e30
      endif
      end

      subroutine ColdDispRelaSol(n20,xh0,thetad0,fpara,
     1 ioneta1,ioneta2,n2,DD33,dDDdx,dDDdkp,dDDdkz)
c version 1.0
c Lunjin Chen, Apr 27th 2009

c xh0 = w normalized to H+ grofrequency
c k normalized to H+ gyrofrequency / c
c if nargin<7, flag_n2pm='n2p';,end;
      use mod_PhyConst
      use mod_WavesWork
	implicit none

	real*8 n20,xh0,thetad0,fpara,ioneta1,ioneta2


	real*8 dn2dx,dn2dkp,dn2dkz,n2	
	real*8 DD33(3,3),dDD33dkz(3,3),dDD33dkp(3,3)
      real*8 dDD33dx(3,3)
	real*8 dDDdx,dDDdkp,dDDdkz,DD
	real*8 dD11dkz,dD12dkz,dD13dkz,dD21dkz,dD22dkz  
	real*8 dD23dkp,dD31dkp,dD32dkp,dD33dkp  
	real*8 dD11dx,dD12dx,dD13dx,dD21dx,dD22dx  
	real*8 dD23dx,dD31dx,dD32dx,dD33dx  
	real*8 dD11dkp,dD12dkp,dD13dkp,dD21dkp,dD22dkp  
	real*8 dD23dkz,dD31dkz,dD32dkz,dD33dkz  
	real*8 D11,D12,D13,D21,D22  
	real*8 D23,D31,D32,D33  

	real*8 x,theta,alphastar,k0,x2,x3
	real*8 eta1,eta2,eta3
      real*8 RL,PRL,PS,PD
	real*8 dLdx,dRdx,dSdx,dPdx,dDdx
	real*8 dRLdx,dPRLdx,dPSdx,dPDdx
	
	real*8 sth4,sth,cth,sth2,cth2
	real*8 dsthdkp,dsthdkz,dsth2dkp,dsth2dkz
	real*8 dcthdkp,dcthdkz,dcth2dkp,dcth2dkz
	real*8 dAdx,dBdx,dCdx
	real*8 F,F2,n2m,n2p
	real*8 n2plus_v1,n2min_v1,n2plus_v2,n2min_v2
      output_dr=0.0d0

      theta=thetad0*rad
      x=xh0

c     ion composition
      eta1=ioneta1
      eta2=ioneta2
      eta3=1.0d0-eta1-eta2

c      memp=emas/hmas
      alphastar=1.0d0/(fpara*fpara)
      x2=x*x
      x3=x2*x

c Define Stix Notation in a multi-ion plasmas (H+-He+-O+)
      L=1.0d0-1.0d0/(alphastar*memp*x)* 
     1      (1.0d0/(memp*x+1)+eta1/(x-1)+eta2/(4*x-1)+eta3/(16*x-1))
      R=1.0d0-1.0d0/(alphastar*memp*x)* 
     1      (1.d0/(memp*x-1)+eta1/(x+1)+eta2/(4*x+1)+eta3/(16*x+1))
      P=1.0d0-1.d0/(alphastar*memp*x2)*
     1      (1.0d0/memp+eta1+1.0d0/4*eta2+1.0d0/16*eta3)
      S=0.5d0*(R+L)
      D=0.5d0*(R-L)
      RL=R*L
      PRL=P*RL
      PS=P*S
      PD=P*D



c Calculate Derivative of Stix Coeff. W.R.T. x
c Note that L, R, P, S, D is a function of x only
c independent of kz and kp
      dLdx=1.0d0/(alphastar*memp*x2)* 
     1      (1.0d0/(memp*x+1)+eta1/(x-1)+eta2/(4*x-1)+eta3/(16*x-1))
     2      -1.0d0/(alphastar*memp*x)* 
     3      (-1.0d0/(memp*x+1)**2*memp-eta1/(x-1)**2*1.0-
     4      eta2/(4*x-1)**2*4.0-eta3/(16*x-1)**2*16.0)

      dRdx=1.0d0/(alphastar*memp*x2)* 
     1      (1.0d0/(memp*x-1)+eta1/(x+1)+eta2/(4*x+1)+eta3/(16*x+1))
     2      -1.0d0/(alphastar*memp*x)* 
     3      (-1.0d0/(memp*x-1)**2*memp-eta1/(x+1)**2-
     4      eta2/(4*x+1)**2*4.0-eta3/(16*x+1)**2*16.0)

      dPdx=+2.0d0/(alphastar*memp*x3)*
     1       (1.0d0/memp+eta1+1.0d0/4*eta2+1.0d0/16*eta3)

      dSdx=0.5d0*(dRdx+dLdx)
      dDdx=0.5d0*(dRdx-dLdx)
      dRLdx=dRdx*L+R*dLdx
      dPRLdx=dPdx*RL+P*dRLdx
      dPSdx=dPdx*S+P*dSdx
      dPDdx=dPdx*D+P*dDdx


      sth=dsin(theta)
      cth=dcos(theta)
      sth2=sth*sth
      cth2=1.0d0-sth2
      sth4=sth2*sth2
c     A-H cold dispersion relation of a biquardratic equation
c     A*n4+B*n2+C=0
c    NOTE the difference from p65 Thorne 250B Solar System plasma physics

      A=S*sth2+P*cth2
      B=-1.0d0*(RL*sth2+PS*(1.0d0+cth2))
      C=PRL

      dAdx=dSdx*sth2+dPdx*cth2
      dBdx=-1.0d0*(dRLdx*sth2+dPSdx*(1.0d0+cth2))
      dCdx=dPRLdx

      F2=(RL-PS)*(RL-PS)*sth4+4.0d0*PD*PD*cth2
      F2=B*B-4.0d0*A*C

      F=dsqrt(F2)

c     n2=(-B+-F)/2A or n2=2C/(B-+F)
      n2min_v1=(-B-F)/(2.0d0*A)
      n2min_v2=2.0d0*C/(-B+F)
      n2plus_v1=(-B+F)/(2.0d0*A)
      n2plus_v2=2.0d0*C/(-B-F)

      n2m=n2min_v1
      n2p=n2plus_v1
      if(n2mdl.eq.'n2m') then
            n2=n2m
      elseif(n2mdl.eq.'n2p') then
            n2=n2p
      elseif(n2mdl.eq.'Lxx') then
            n2=L
      elseif(n2mdl.eq.'Rxx') then
            n2=R
      elseif(n2mdl.eq.'n20') then 
            n2=n20
      else
            n2=n20;
c      diff1_n2=abs(n2m-n2);
c      diff2_n2=abs(n2p-n2);
c      mindiff=min([min(diff1_n2(:)),min(diff2_n2(:))]);
c      if(mindiff<0.05*n2)
c        display('warning: diff in n2 is found betwwen from HOTRAY and ColdDispRelaSol_1d');
c      end if
      end if


c 
c if(nions==1)
c     if(mode==3)
c         ind3=find(x>0 & x<1);n2(ind3)=n2p(ind3);
c     elseif(mode==4)
c         ind4=find(x>0 & x<1);n2(ind4)=n2m(ind4);
c     else
c         
c     end
c end
c 
c 
c if(nions==3)
c     if(mode==3)
c         ['mode: ',num2str(mode)]
c         ind31=find(x>1./4 & x<1 & n2p >0 & L>=R & L>0);n2(ind31)=n2p(ind31);
c         ind32=find(x>1./4 & x<1 & n2p >0 & L<R & L>0);n2(ind32)=n2p(ind32);
c     elseif(mode==4)
c         ['mode: ',num2str(mode)]
c         ind41=find(x>1./4 & x<1 & n2p >0 & L<R & L>0);n2(ind41)=n2m(ind41);
c         ind42=find(x>1./4 & x<1 & n2p >0 & L>=R & L>0);n2(ind42)=n2m(ind42);
c         ind43=find(x>1./16. & x<1./4 & n2p >0 & L<R & L>0);n2(ind43)=n2p(ind43);
c         ind44_1=find((x>=1./4. & x<1. & L<0));n2(ind44_1)=n2m(ind44_1);
c         ind44_2=find(x<1./4 & x > 1./16 & L>R & L>0);n2(ind44_2)=n2p(ind44_2);
c     elseif(mode==5)
c         ['mode: ',num2str(mode)]
c         ind52=find(x<1./4 & x>1./16 & L>R & L>0); n2(ind52)=n2m(ind52);
c         ind51_1=find(x<1./4 & x>1./16 & L<R);n2(ind51_1)=n2m(ind51_1);
c         ind51_2=find(x<1./16);n2(ind51_2)=n2p(ind51_2);
c     elseif(mode==6)
c         ['mode: ',num2str(mode)]
c         ind62=find(x<1./16);n2(ind62)=n2m(ind62);
c     elseif(mode==7)
c         ['mode: ',num2str(mode)]
c         ind32=find(x>1./4 & x<1 & n2p >0 & L<R & L>0);n2(ind32)=n2p(ind32);
c         ind42=find(x>1./4 & x<1 & n2p >0 & L>=R & L>0);n2(ind42)=n2m(ind42);
c         ind52=find(x<1./4 & x>1./16 & L>R & L>0); n2(ind52)=n2m(ind52);
c         ind62=find(x<1./16);n2(ind62)=n2m(ind62);
c     elseif(mode==20)
c         ['mode: ',num2str(mode)]
c 	  n2(:)=n2m(:);
c     elseif(mode==30)
c         ['mode: ',num2str(mode)]
c 	  n2(:)=n2p(:);
c     else
c 	
c         
c     end
c end
c 



c calculate derivatives
c     x=w/w0; % kp0=kp c/w0; kz0=kz c /w0
      dn2dx=n2*(-2.0d0)/x 
      dn2dkp=2.0d0*dsqrt(n2)/x*sth
      dn2dkz=2.0d0*dsqrt(n2)/x*cth
      k0=dsqrt(n2)*x

      dsthdkp=cth*cth/k0
      dsthdkz=-cth*sth/k0
      dcthdkp=-sth*cth/k0
      dcthdkz=sth*sth/k0

      dsth2dkp=2.0d0*sth*dsthdkp
      dsth2dkz=2.0d0*sth*dsthdkz
      dcth2dkp=2.0d0*cth*dcthdkp
      dcth2dkz=2.0d0*cth*dcthdkz


      D11=2.0d0*R-2.0d0*n2+n2*sth2
      dD11dx=2.0d0*dRdx-2.0d0*dn2dx+dn2dx*sth2
      dD11dkp=-2.0d0*dn2dkp+dn2dkp*sth2+n2*dsth2dkp
      dD11dkz=-2.0d0*dn2dkz+dn2dkz*sth2+n2*dsth2dkz

      D12=n2*sth2
      dD12dx=dn2dx*sth2
      dD12dkp=dn2dkp*sth2+n2*dsth2dkp
      dD12dkz=dn2dkz*sth2+n2*dsth2dkz


      D13=n2*cth*sth
      dD13dx=dn2dx*cth*sth
      dD13dkp=dn2dkp*cth*sth+n2*dcthdkp*sth+n2*cth*dsthdkp
      dD13dkz=dn2dkz*cth*sth+n2*dcthdkz*sth+n2*cth*dsthdkz


      D21=D12
      dD21dx=dD12dx
      dD21dkp=dD12dkp
      dD21dkz=dD12dkz

      D22=2.0d0*L-2.0d0*n2+n2*sth2
      dD22dx=2.0d0*dLdx-2.0d0*dn2dx+dn2dx*sth2
      dD22dkp=-2.0d0*dn2dkp+dn2dkp*sth2+n2*dsth2dkp
      dD22dkz=-2.0d0*dn2dkz+dn2dkz*sth2+n2*dsth2dkz

      D23=n2*cth*sth
      dD23dx=dn2dx*cth*sth
      dD23dkp=dn2dkp*cth*sth+n2*dcthdkp*sth+n2*cth*dsthdkp
      dD23dkz=dn2dkz*cth*sth+n2*dcthdkz*sth+n2*cth*dsthdkz

      D31=D13
      dD31dx=dD13dx
      dD31dkp=dD13dkp
      dD31dkz=dD13dkz


      D32=D23
      dD32dx=dD23dx
      dD32dkp=dD23dkp
      dD32dkz=dD23dkz

      D33=P-n2*sth2
      dD33dx=dPdx-dn2dx*sth2
      dD33dkp=-dn2dkp*sth2-n2*dsth2dkp
      dD33dkz=-dn2dkz*sth2-n2*dsth2dkz
      
      DD= D11*(D22*D33-D32*D23) 
     1   -D12*(D21*D33-D23*D31) 
     2   +D13*(D21*D32-D22*D31)

      dDDdx=dD11dx*(D22*D33-D32*D23) 
     1     +D11*(dD22dx*D33+D22*dD33dx-dD32dx*D23-D32*dD23dx) 
     2     -dD12dx*(D21*D33-D23*D31) 
     3     -D12*(dD21dx*D33+D21*dD33dx-dD23dx*D31-D23*dD31dx) 
     4     +dD13dx*(D21*D32-D22*D31) 
     5     +D13*(dD21dx*D32+D21*dD32dx-dD22dx*D31-D22*dD31dx)

      dDDdkp=dD11dkp*(D22*D33-D32*D23) 
     1	+D11*(dD22dkp*D33+D22*dD33dkp-dD32dkp*D23-D32*dD23dkp) 
     2      -dD12dkp*(D21*D33-D23*D31) 
     3      -D12*(dD21dkp*D33+D21*dD33dkp-dD23dkp*D31-D23*dD31dkp) 
     4      +dD13dkp*(D21*D32-D22*D31) 
     5      +D13*(dD21dkp*D32+D21*dD32dkp-dD22dkp*D31-D22*dD31dkp)

      dDDdkz=dD11dkz*(D22*D33-D32*D23) 
     1	+D11*(dD22dkz*D33+D22*dD33dkz-dD32dkz*D23-D32*dD23dkz) 
     2      -dD12dkz*(D21*D33-D23*D31) 
     3      -D12*(dD21dkz*D33+D21*dD33dkz-dD23dkz*D31-D23*dD31dkz)
     4      +dD13dkz*(D21*D32-D22*D31)
     5      +D13*(dD21dkz*D32+D21*dD32dkz-dD22dkz*D31-D22*dD31dkz)

      DD33(1,1)=D11
	DD33(1,2)=D12
	DD33(1,3)=D13
	
      DD33(2,1)=D21
	DD33(2,2)=D22
	DD33(2,3)=D23
	
      DD33(3,1)=D31
	DD33(3,2)=D32
	DD33(3,3)=D33
	
      dDD33dx(1,1)=dD11dx
	dDD33dx(1,2)=dD12dx
	dDD33dx(1,3)=dD13dx
	
      dDD33dx(2,1)=dD21dx
	dDD33dx(2,2)=dD22dx
	dDD33dx(2,3)=dD23dx
	
      dDD33dx(3,1)=dD31dx
	dDD33dx(3,2)=dD32dx
	dDD33dx(3,3)=dD33dx
	
      dDD33dkp(1,1)=dD11dkp
	dDD33dkp(1,2)=dD12dkp
	dDD33dkp(1,3)=dD13dkp
	
      dDD33dkp(2,1)=dD21dkp
	dDD33dkp(2,2)=dD22dkp
	dDD33dkp(2,3)=dD23dkp
	
      dDD33dkp(3,1)=dD31dkp
	dDD33dkp(3,2)=dD32dkp
	dDD33dkp(3,3)=dD33dkp
	
      dDD33dkz(1,1)=dD11dkz
	dDD33dkz(1,2)=dD12dkz
	dDD33dkz(1,3)=dD13dkz
	
      dDD33dkz(2,1)=dD21dkz
	dDD33dkz(2,2)=dD22dkz
	dDD33dkz(2,3)=dD23dkz
	
      dDD33dkz(3,1)=dD31dkz
	dDD33dkz(3,2)=dD32dkz
	dDD33dkz(3,3)=dD33dkz

      output_dr(1)=n2
      output_dr(2)=n20
      output_dr(3)=n2m
      output_dr(4)=n2p
      output_dr(5)=L
      output_dr(6)=R
      output_dr(7)=thetad0
      output_dr(8)=n2min_v1
      output_dr(9)=n2plus_v1
      output_dr(10)=n2min_v2
      output_dr(11)=n2plus_v2
      output_dr(12)=xh0
      output_dr(13)=thetad0
      output_dr(14)=fpara
      output_dr(15)=ioneta1
      output_dr(16)=ioneta2
      output_dr(17)=A
      output_dr(18)=B
      output_dr(19)=C
      output_dr(20)=R
      output_dr(21)=L
      output_dr(22)=P
      output_dr(23)=S
      output_dr(24)=D
      output_dr(25)=F
      output_dr(26)=DD
      output_dr(27)=sth
      output_dr(28)=cth
      output_dr(29)=-dDDdkz/dDDdx*clight !  parallel group speed
	output_dr(30)=-dDDdkp/dDDdx*clight !  perpendicular group speed
      end subroutine

