      subroutine LocalGrowthRate(ioneta1,ioneta2,
     1 OmegaH0,wpe0,x,thetad,n2i,ki,gamma1)
c  subroutine to calculate local growth rate     
c  assuming (1) much smaller hot ion number density, 
c		    compared with the cold species number density
c Lunjin Chen, April 28rd, 2009
c     input argument
c     -ioneta1 : H+ composition, e.g., 0.77
c     -ioneta2 : He+ composition, e.g., 0.2
c     -OmegaH0 : H+ gyro angular frequency, rad/s
c     -wpe0    : e- plasma angular frequency, rad/s
c     -x       : wave frequeny normalized to H+ gyrofrequency
c     -thetad  : wave normal angle, deg
c     -n2i     : for HOTRAY input
c     *** output ***
c     -ki      : convective growth rate, m^-1
c     -gamma1  : temporal growth rate, s-1

c     before calling this subroutine
c     1) specified species by their mass and charge
c     2) vp grids
c     3) parallel resonant velocity lower and upper limits
c     4) set up the PSD for specified species
c     5) specified the harmonic range of resonances
c     PIG_INIT
c     PIG_SETUP(iq,imas,vzmin,vzmax,vpmin,vpmax,'lin',nvpnew)
c	dfvvpa_data_setup(nVVnew,nPAnew,VVARRnew,PAARRnew,LNPSD2dnew,
c                 MASKnew,psdsclnew)
c	LGR_SETUP('dfdata','n2m','spe',ixh,ixh)
c     
c    
c     other comments
c     -principal cyclotron resonance for R-mode with e- when m=-1     
c     -principal cyclotron resonance for L-mode with H+ when m=1     
c     -integration along the bessel function argument kp*vp/Omega
c           instead of the vp, pending
c 
c     
c     
      use mod_PhyConst
      use mod_WavesWork
      implicit none


      integer*4 tid
      parameter(tid=73)

c     input variable     
	integer*4 ispec
      integer*4 i,j
	real*8 ioneta1,ioneta2,OmegaH,wpe,x,thetad,n2,wpe0
      integer*4 outputarr
      real*8 fpara
      real*8 OmegaH0
      real*8 commonfactor
      real*8 sth,cth,sth2,sth4,cth2
      integer*4 mstart_try,mend_try
      
      character allfluxfile*100  

c	local variable
      character casestr*7,ionetastr*9,modestr*4
      real*8 psdf
	real*8 Omegae,WP0,wpi2
	real*8 xfo,theta,dvp
	integer*4 im,ivp,mm
	real*8 n2i,D3(3,3),D3new(3,3)
	real*8 dDDdx,dDDdkp,dDDdkz
	real*8 vresm,kz,rOmegai
	real*8 dFdvp_resm,dFdvz_resm
	real*8 dFdPA,G1,G2vp
	real*8 IF11,IF12,IF13
	real*8 IF21,IF22,IF23
	real*8 IF31,IF32,IF33
	real*8 I33m(3,3,NMMAX),IDm(NMMAX),ID
	real*8 gamma0,gamma1,ki
	real*8 Jm,Jmp,Jmn,zeta
	real*8 k0fac,ki_XT,kk,kp  
	integer IFALL
	real*8 vratio,vgp,vgz,vp,vphz,vg
      real*8 iq,imas
      integer*4 icase(4)

      real*8 dbesjn
      integer*4 flag_n2
	integer*4 ifailflag
      character*100 strtmp
      real*8 ID_K66,ID_K66m(NMMAX)      
      real*8 xL,xR,xP,xS,xcs
      real*8 G1dvp, G2vpdvp
      real*8 G1fac,G2vpfac

      

      output_gr=0.0d0
      ifailflag=0

            
      if (OmegaH0.lt.0) then
c            OmegaH=0.31e-4/THISrL**3*ev/hmas
      else
            OmegaH=OmegaH0      
      end if
      OmegaH0=OmegaH
      Omegae=OmegaH/memp

      if (wpe0.lt.0) then
              fpara=-1.0d0*wpe0
              wpe=fpara*Omegae
      else
            wpe=wpe0
	      fpara=wpe/Omegae
      end if
      wpe0=wpe
c  plasma frequ in rad per second for 1 m-3 H+
      WP0=dsqrt(1.0d0/(hmas*ee0))*ev

c     species information
      iq=THISiq   ! in the unit of element charge
      imas=THISimas     ! in the unit of H+ atmos mass
    	rOmegai=iq/imas !the species gyro-frequ normalized to H+
    	wpi2=WP0**2*iq**2/imas !species plasma frequ normalized to H+ with no. den 1 m-3 

c wave properties 
      theta=thetad*rad

c     need to set n2i less than zero if not n2 as input
c     need info about n2,D3,dDDdx,dDDdkp,dDDdkz
c     need a way to select one of n2 from two different wave mode
	call ColdDispRelaSol(n2i,x,
     1    thetad,fpara,ioneta1,ioneta2,
     2    n2,D3,dDDdx,dDDdkp,dDDdkz)

	if (n2 .lt. 0) then
		ki=-1d31
		gamma0=-1d31
		gamma1=-1d31
		ifailflag=1
		GO TO 999
	end if


	k0fac=clight/OmegaH !normalized factor of K used in ColdDispRelaSol
	kk=dsqrt(n2)*x/k0fac
      sth=dsin(theta)
      cth=dcos(theta)
      sth2=sth*sth
      cth2=cth*cth
      sth4=sth2*sth2
	kp=kk*sth
	kz=kk*cth ! include k sign, depending on the value of theta
	vphz=x*OmegaH/kz ! parallel phase speed


      if(n2mdl.eq.'n20') then
            vgz=input_gr(1)
            vgp=input_gr(2)
            if(vgz.lt.(-1e30).and.vgp.lt.(-1e30))then
      	vgz=-dDDdkz*k0fac/dDDdx*OmegaH !; % parallel group speed
	      vgp=-dDDdkp*k0fac/dDDdx*OmegaH !; % perpendicular group speed
            endif
      else
      	vgz=-dDDdkz*k0fac/dDDdx*OmegaH !; % parallel group speed
	      vgp=-dDDdkp*k0fac/dDDdx*OmegaH !; % perpendicular group speed
      endif

      commonfactor=2*pi*pi/(x*OmegaH*abs(kz))/dDDdx*wpi2


c ********************************************************
c     mstart and mend is for setting up mstart_try and mend_try
c mrange=[mstart:mend]; % harmonic number m range to be considerred
      if (mresmdl.eq.'def') then  ! default value
            mstart=MH0
            mend=MH1
      elseif (mresmdl.eq.'aut') then ! automatically calculated
            vratio=THISVZRESMAX/abs(vphz)
c           mend always greater than mstart
            mstart=floor(x*(-vratio+1)) ! % min m
            mend=ceiling(x*(vratio+1)) ! % max m
      elseif (mresmdl.eq.'spe') then ! specified
c            mstart=para_gr(9)
c            mend=para_gr(10)
      else
      endif


	nm=mend-mstart+1
      if(nm.gt.400) then
              write(*,*) 'warning: nm>400!:',nm
              write(*,*) 'change in m:', mstart,mend,nm
              mstart=(mstart+mend)/2-200
              mend=400+mstart-1
              nm=mend-mstart
	end if

c ********************************************************



      ID_K66m=0.0d0
      ID_K66=0.0d0


    	im=0;
      mstart_try=mstart
      mend_try=mend
    	do 30 mm=mstart_try,mend_try 
      vresm=OmegaH/kz*(x-mm*rOmegai)! resonant velocity at m-order harmonics
      if(abs(vresm).gt.THISVZRESMAX) go to 30
      im=im+1
c     update mstart and mend      
      if(im.eq.1) mstart=mm
      mend=mm
      vresmarr(im)=vresm
      
c     reset VPARR and DVPARR here, based on the kp/Omega
c     what if kp=0

	do 50 ivp=1,NVP
	  vp=VPARR(ivp)
	  dvp=DVPARR(ivp)

c     Jm,Jmp,Jmn can be calculated before handed        
c     zeta can be negative value for electron
        zeta=(vp*kp/(rOmegai*OmegaH))
        Jm=dbesjn(mm,zeta)
        Jmp=dbesjn(mm+1,zeta)
        Jmn=dbesjn(mm-1,zeta)
      

c     derivatives of F w.r.t. vp and vz        
	call PSD_deriv(vresm,vp,psdf,dFdvz_resm,dFdvp_resm)

        dFdPA=dFdvp_resm*vresm-dFdvz_resm*vp 
        G1=dFdvp_resm-dFdPA/vphz
        G2vp=(dFdvz_resm*vp+dFdPA*(mm*rOmegai)/x)*Jm
        G1dvp=G1*dvp
        G2vpdvp=G2vp*dvp
c     mm*rOmegai/x => m Omega_i / omega        

c     Integration over vp        
        xL=L-n2
        xR=R-n2
        xP=P-n2*sth2
        xS=S-n2
        xcs=cth*sth
      G1fac=vp*( xP*vp*( 2*xL*Jmp*Jmp
     1                +2*xR*Jmn*Jmn 
     2                +n2*sth2*(Jmp-Jmn)**2 )
     3        -(n2*xcs)*( 2*vresm*Jm*(Jmp*xR+Jmn*xL)          
c    original formula below     
c     3        -(n2*xcs)*( 2*vresm*Jm*(Jmp*xL+Jmn*xR)          
     4                   +n2*xcs*vp*(Jmp-Jmn)**2 ) )
      G2vpfac=( 4*vresm*Jm*(xL*xR+n2*sth2*xS)
     1        -2*n2*xcs*vp*(xR*Jmn+xL*Jmp)  ) 
      ID_K66m(im)=ID_K66m(im)+G1dvp*G1fac+G2vpdvp*G2vpfac

c *******************************************
c     1 : dfdvz      
c     2 : dfdvp      
c     3 : G1      
c     4 : G2 x vp      
c     5 : W(G1)
c     6 : W(G2 x vp)
c     7 : W(dfdvz)
c     8 : W(dfdvp)
      LGR_WT(ivp,im,1)=dFdvz_resm
      LGR_WT(ivp,im,2)=dFdvp_resm
      LGR_WT(ivp,im,3)=G1
      LGR_WT(ivp,im,4)=G2vp
      LGR_WT(ivp,im,5)=G1fac*commonfactor
      LGR_WT(ivp,im,6)=G2vpfac*commonfactor
      LGR_WT(ivp,im,7)=commonfactor*
     1  (G1fac*(vp/vphz)+G2vpfac*vp*Jm*(1-mm*rOmegai/x))
      LGR_WT(ivp,im,8)=commonfactor*
     1  (G1fac*(1-vresm/vphz)+G2vpfac*Jm*vresm*mm*rOmegai/x)
c *******************************************

  50	continue  ! for ivp
            ID_K66=ID_K66+ID_K66m(im)
  30  continue ! % for im
c     update nm       
	nm=mend-mstart+1

c assume ki has the same direction of k
c positive gamma0 and negative ki => growth
c negative gamma0 and positive ki => damping
	gamma0=commonfactor*ID_K66 !% gamma*dD/dw
	gamma1=gamma0*OmegaH !% temporal growth rate  
      vg=dsqrt(vgp*vgp+vgz*vgz)
	ki=-gamma1/vg ! % ki.vg/|vg|
	gammamarr(1:nm)=commonfactor*OmegaH*ID_K66m(1:nm)
	kimarr(1:nm)=-gammamarr(1:nm)/dsqrt(vgp*vgp+vgz*vgz)
      output_gr(1)=vgz
      output_gr(2)=vgp
      output_gr(3)=OmegaH
      output_gr(4)=wpe
      output_gr(5)=kz
      output_gr(6)=kp
      output_gr(7)=x
      output_gr(8)=thetad
      output_gr(9)=vg
      output_gr(10)=OmegaH/vg
  999	continue	
      output_gr(99)=ifailflag

      if (LGR_testmode.eq.1.and.ifailflag.eq.0) then
            open(unit=tid,file=trim(LGR_testfile),
     1	form='binary',status='unknown')
              write(tid) nm,mstart,mend,NVP
c              write(tid) x,thetad,kz,kp,OmegaH,wpe,vgz,vgp
              write(tid) output_gr(1:8)
              write(tid) ki,gamma0,gamma1
              write(tid) vresmarr(1:nm),
     1         gammamarr(1:nm),kimarr(1:nm)
              write(tid) VPARR(1:NVP),DVPARR(1:NVP)
cc segmentation fault if write(tid) LGR_WT(:,1:nm,1:8)
              do i=1,8
              do j=1,nm
                  write(tid) LGR_WT(1:NVP,j,i)
              enddo
              enddo
              close(tid)
      endif
	end subroutine
