      module mod_PhyConst
	implicit none
c IS unit (mks)
      real*8 pi,ev,emas,hmas,ee0,mu0,clight,kb,rad
      real*8 dBfac, RE, memp
      parameter (pi=3.14159265358979324d0)
      parameter (ev=1.60210d-19,emas=9.10908d-31,hmas=1.67252d-27)
      parameter (memp=emas/hmas)
      parameter (ee0=8.85442d-12,mu0=4.0d0*pi*1.0d-7)
c	ee0->eps0	  
      parameter (clight=2.99792501d8)
      parameter (kb=1.38054d-23)
      parameter (rad=pi/180.0d0)
      parameter (dBfac=8.6859d0)
      parameter (RE=6370d3)
	real*8 BBeq
	parameter (BBeq=0.31d-4)
	real*8 Hfac,efac,Hefac,Ofac  
	parameter(
     1	Hfac=5.4490d-19,
     2	efac=1.6163d-25,
     3	Hefac=8.7185d-18,
     4	Ofac=1.3950d-16)
	end module

	function den2fpe(dene)
c	electron density -> electron plasma frequency 
	use mod_PhyConst
	implicit none
	real*8 dene,den2fpe
	den2fpe=sqrt(dene/emas/ee0)*ev/(2*pi)
	end function  

	function den2fph(denh)
c	proton density -> proton plasma frequency 
	use mod_PhyConst
	implicit none
	real*8 denh,den2fph
	den2fph=sqrt(denh/hmas/ee0)*ev/(2*pi)
	end function  

	function fpe2den(fpe)
c	electron plasma frequency -> electron density  
	use mod_PhyConst
	implicit none
	real*8 fpe,fpe2den
	fpe2den=(2*pi*fpe)**2*emas*ee0/(ev**2)
	end function  

	function fph2den(fph)
c	proton plasma frequency -> proton density  
	use mod_PhyConst
	implicit none
	real*8 fph,fph2den
	fph2den=(2*pi*fph)**2*hmas*ee0/(ev**2)
	end function  

	function b2fce(b)
c	magnetic field strength -> electron gyro frequency  
	use mod_PhyConst
	implicit none
	real*8 b2fce,b
	b2fce=ev*b/emas/(2*pi)
	end function

	function b2fch(b)
c	magnetic field strength -> proton gyro frequency  
	use mod_PhyConst
	implicit none
	real*8 b2fch,b
	b2fch=ev*b/hmas/(2*pi)
	end function

	function fce2b(fce)
c	electron gyro frequency -> magnetic field strength 
	use mod_PhyConst
	implicit none
	real*8 fce2b,fce
	fce2b=(2*pi*fce)*emas/ev
	end function

	function fch2b(fch)
c	proton gyro frequency -> magnetic field strength 
	use mod_PhyConst
	implicit none
	real*8 fch2b,fch
	fch2b=(2*pi*fch)*hmas/ev
	end function

	function n2wp(dene,imas)
c	density -> plasma frequency  
	use mod_PhyConst
	real*8 dene,n2wp
	integer*4 imas
	if (imas.eq.0) then
		n2wp=dsqrt(dene*ev*ev/(ee0*emas))
	else
		n2wp=dsqrt(dene*ev*ev/(ee0*imas*hmas))
	endif

	end function n2wp

      subroutine linspace(xarr,x0,x1,nx)
      real*8 xarr(*)
      real*8 x0,x1,dx
      integer*4 nx,i
      dx=(x1-x0)/(nx-1)
      do i=1,nx
            xarr(i)=x0+(i-1)*dx
      enddo
      end subroutine linspace
      subroutine logspace(xarr,x0,x1,nx)
      real*8 xarr(*)
      real*8 x0,x1,xfac
      integer*4 nx,i
      xfac=(x1/x0)**(1.0/(nx-1))
      do i=1,nx
            xarr(i)=x0*xfac**(i-1)
      enddo
      end subroutine logspace

	
