*<TITLE>CVT_DATE.FOR Algorithms for doing date calculations with 4 digit years.    </TITLE>
* TECHNICAL: Algorithms for doing date calculations with 4 digit years.
*
* Gathered by: Harold Zbiegien
*              American Greetings Corp.
*       email: bb206@Cleveland.Freenet.Edu
*
*
* These are several routines I have found by searching the Web, SIMTEL
* archives and relevant books and magazine articles.  My preference was
* to use routines that did NOT utilize tables.  All routines have been
* checked by me against each other and the SAS date functions in MVS SAS
* Release 6.08 level TS404.  This level of SAS is only good for dates in
* the range: Jan 1, 1600< date <Feb 29, 4000.  Incorrect answers result
* if dates outside of this range are used.  The routines below have been
* tested by me for dates in the range: Oct 14, 1582 < date < Dec 31, 9999.
*   ==> Disclaimer:
*   ==>  I make no warrantee or guarantee on the accuracy of these
*   ==>  routines or your implementation into actual code.
*   ==>  I speak for myself, my statements are not those of my company.
*
* Note the Gregorian calendar, on which all these routines are based
* assumes the year is 365.2425 days long.  This is not true.  One
* astronomer put the figure at 365.24219879 days at the start of the
* 1900 epoch.  ALSO the length of the year is gradually SHORTENING!
* Around the year 4317 we will have to drop a day, and again somewhere
* near the years 6000, 7500, and 9000.  (reference: "The Gregorian
* calendar", G. Moyer, Scientific American, May 1982)
*
* These routines should be implemented using 32 bit or 9 digit precision
* arithmetic in the computer language of your choice.  In some cases you
* can get by with 16 bit precision but I'll leave that up to you, the
* implementor, to figure out where.  All division is integer division.
* By this I mean you drop the remainder.  I.E. 35 / 4 is 8.  The routines
* makes use of the MOD function.  This is the remainder after integer
* division.  I.E.  MOD(35,4) is 3.  To make division even more clear I use
* the INT function, integer portion, or truncate. I.E. INT(35 / 4) is 8.
*
* Routine A:  Convert from YYYY,MM,DD to YYYY,NNN and day-of-week
* Routine B:  Convert from YYYY,NNN to YYYY,MM,DD and day-of-week
* Routine C:  Convert from YYYY,NNN to Lilian (day 1=Fri Oct 15, 1582)
* Routine D:  Convert from Lilian to YYYY,NNN
* Routine E:  Convert from Lilian to day-of-week, and notes on the use
*             of Lilian dates and day-of-week
* Routine F:  Convert from YYYY,MM,DD to YYYY,NNN
* Routine G:  Convert from YYYY,MM,DD to day-of-week

* Fortran definitions:
*      SUBROUTINE cvt_date_a(yyyy,mm,dd,nnn,dow)
*      SUBROUTINE cvt_date_b(yyyy,mm,dd,nnn,dow)
*      SUBROUTINE cvt_date_c(yyyy,nnn,lil)
*      SUBROUTINE cvt_date_d(yyyy,nnn,lil)
*      SUBROUTINE cvt_date_e(lil,dow)
*      SUBROUTINE cvt_date_e(lil,dow)
*      SUBROUTINE cvt_date_f(yyyy,mm,dd,nnn)
*      SUBROUTINE cvt_date_g(yyyy,mm,dd,dow)
* C callers: All arguments are int * ! 
* C Prototypes:
*      void  cvt_date_a(int *yyyy,int *mm,int *dd,int *nnn,int *dow)
*      void  cvt_date_b(int *yyyy,int *mm,int *dd,int *nnn,int *dow)
*      void  cvt_date_c(int *yyyy,int *nnn,int *lil)
*      void  cvt_date_d(int *yyyy,int *nnn,int *lil)
*      void  cvt_date_e(int *lil,int *dow)
*      void  cvt_date_e(int *lil,int *dow)
*      void  cvt_date_f(int *yyyy,int *mm,int *dd,int *nnn)
*      void  cvt_date_g(int *yyyy,int *mm,int *dd,int *dow)


* There are other routines for converting to/from Julian dates, the
* Julian date that astronomers use, where day zero was Jan 1, 4713 BC.
* But I felt that they were not of use to me in standard business
* functions.
*
* THE ALGORITHMS
*
* ROUTINE A:  Convert from YYYY,MM,DD to YYYY,NNN and day-of-week
*   Reference: Algorithm 398 in the Oct 1970 Communications of the ACM
*   By Richard A. Stone.  The day-of-week calculations were added by me.
*
*  Input:  YYYY, MM, DD     Output:  NNN (1-366)
* EX: 1996, Nov 1 results in NNN = 306, DOW = 5

      SUBROUTINE cvt_date_a(yyyy,mm,dd,nnn,dow)
      IMPLICIT NONE
      INTEGER yyyy,mm,dd,nnn,dow
      INTEGER ly,t
      IF (MOD(YYYY,4)   .eq. 0) THEN
        LY = 1
      ELSE
        LY = 0
      ENDIF
      IF (MOD(YYYY,100) .eq. 0) THEN
        LY = 0
      ENDIF
      IF (MOD(YYYY,400) .eq.0) THEN
        LY = 1    !LY is 1 if it is a leap year
      ENDIF
      NNN = INT(((MM + 2) * 3055) / 100) + DD - 91
      IF (NNN .GT. (59 + LY)) THEN
        NNN = NNN - 2 + LY
      ENDIF
* for day-of-week (0-6, 0=Sunday, 1=Monday, etc.)
      T = INT(YYYY / 100) - 6 - INT(YYYY / 400)
      DOW = MOD((NNN + INT((YYYY * 5) / 4) - LY - T),7)
      END

*ROUTINE B:  Convert from YYYY,NNN to YYYY,MM,DD and day-of-week
*  Reference: This is a reversal of Routine A worked out by me.

* Input:   YYYY, NNN (1-366)  Output: MM, DD
* EX: 1996, 306 results in Nov 1, DOW = 5

      SUBROUTINE cvt_date_b(yyyy,mm,dd,nnn,dow)
      IMPLICIT NONE
      INTEGER yyyy,mm,dd,nnn,dow
      INTEGER work,ly,t
      IF (MOD(YYYY,4)   .eq. 0) THEN
        LY = 1
      ELSE
        LY = 0
      ENDIF
      IF (MOD(YYYY,100) .eq. 0) THEN
        LY = 0
      ENDIF
      IF (MOD(YYYY,400) .eq.0) THEN
        LY = 1    !LY is 1 if it is a leap year
      ENDIF
      WORK = NNN
      IF (WORK .GT. (LY + 59)) THEN
        WORK = WORK + 2 - LY
      ENDIF
      MM = INT(((WORK + 91) * 100) / 3055)
      DD = (WORK + 91) - INT((MM * 3055) / 100)
      MM = MM - 2
* for day-of-week (0-6, 0=Sunday, 1=Monday, etc.)
      T = INT(YYYY / 100) - 6 - INT(YYYY / 400)
      DOW = MOD((NNN + INT((YYYY * 5) / 4) - LY - T),7)
      END


*ROUTINE C:  Convert from YYYY,NNN to Lilian (day 1=Fri Oct 15, 1582)
*  Reference: "Computer processing of dates outside the twentieth
*  century" By B. G. Ohms, IBM Systems Journal, vol 25 no 2, 1986.
*  This routine matches the base algorithm in IBM's CEEDAYS routine in
*  their Language Environment implementation as best as I can figure.
*  This routine and CEEDAYS both have the same starting date.
*
* Input: YYYY, NNN (1-366)  Output: LIL (1 = Friday Oct 15, 1582)
* EX: 1996,306 results in LIL = 151229
* EX: May 16, 1988, NNN = 137, results in LIL = 148138

      SUBROUTINE cvt_date_c(yyyy,nnn,lil)
      IMPLICIT NONE
      INTEGER yyyy,nnn,lil
      LIL = INT(((YYYY - 1201) * 36525) / 100) -  139444 + NNN -
     &  INT((YYYY - 1201) / 100)+    INT((YYYY - 1201) / 400)
      END

*
*ROUTINE D:  Convert from Lilian to YYYY,NNN
*  Reference: "Computer processing of dates outside the twentieth
*  century" By B. G. Ohms, IBM Systems Journal, vol 25 no 2, 1986.
*  This routine matches the base algorithm in IBM's CEEDATE routine in
*  their Language Environment implementation as best as I can figure.
*  This routine and CEEDATE both have the same starting date.
*
* Input: LIL (1 = Friday Oct 15, 1582)  Output: YYYY, NNN (1-366)
* EX: 151229 results in YYYY = 1996, NNN = 306

      SUBROUTINE cvt_date_d(yyyy,nnn,lil)
      IMPLICIT NONE
      INTEGER yyyy,nnn,lil
      INTEGER cld,work
      CLD = INT(((LIL + 139444) * 100) / 3652425)
      NNN = CLD + LIL + 139444 - INT(CLD / 4)
      WORK = INT((NNN * 100) / 36525)
      IF (MOD((NNN * 100), 36525) .EQ. 0) THEN
        WORK = WORK - 1
      ENDIF
*   (in some computer languages the 2 lines above can be implemented as
*     divide 36525 into (NNN * 100) giving WORK remainder REM
*    if REM = 0 then WORK = WORK - 1)
      NNN = NNN - INT((WORK * 36525) / 100)
      YYYY = WORK + 1201
      END

*ROUTINE E:  Convert from Lilian to day-of-week, and notes on the use
*            of Lilian dates and day-of-week
* Input: LIL (1 = Friday Oct 15, 1582)
* Output: day-of-week (0-6, 0=Sunday, 1=Monday, etc.)
* EX: 151229 (Nov 1, 1996, 306) results in DOW = 5

      SUBROUTINE cvt_date_e(lil,dow)
      IMPLICIT NONE
      INTEGER lil,dow
      DOW = MOD((LIL + 4),7)
      END


* Notes on day-of-week
*   These routines are coded to return a 0 value for Sunday.  If you
*   wish to have zero represent, say Monday, just add 1 to the value
*   inside the MOD statement.  Other constant values added inside the
*   MOD statement will equate zero to other days.
* Notes on the use of Lilian days.
*   Once you have a Lilian day value you can easily add and subtract
*   constant day values to answer questions like, what day will it be
*   30 days from now?  Or how many days are between two dates?  Lilian
*   days will NOT help you in questions like, how many business days are
*   there between 2 dates?  Things like Holidays, weekends, etc. are NOT
*   accounted for.  I think that only some type of Calendar (table) and
*   a loop thru that table will be able to answer questions like this.
*   Also they will not help in answering questions like what will the
*   date be 15 months from now?, or what is the date on the 3rd Tuesday
*   in September of next year?
*
*ROUTINE F:  Convert from YYYY,MM,DD to YYYY,NNN
*  Reference: "The Year 2000 Computing Crisis" by J. T. Murray and
*   and M. J. Murray.  Documentation on an RPG routine in their
*   diskette that is part of their book.  Most of their other code
*   is table based.
* EX: Nov 1, 1996 results in NNN = 306

      SUBROUTINE cvt_date_f(yyyy,mm,dd,nnn)
      IMPLICIT NONE
      INTEGER yyyy,mm,dd,nnn
      INTEGER ly
      IF (MOD(YYYY,4)   .eq. 0) THEN
        LY = 1
      ELSE
        LY = 0
      ENDIF
      IF (MOD(YYYY,100) .eq. 0) THEN
        LY = 0
      ENDIF
      IF (MOD(YYYY,400) .eq.0) THEN
        LY = 1    !LY is 1 if it is a leap year
      ENDIF
      NNN = INT(3 / (MM + 1)) * (31 * (MM -1) + DD) +
     &  INT((MM + 9) / 12) * (INT(((305 * (MM - 1) - 15) +
     &  INT((MM + 3) / 12) * 5 * INT(18 / MM)) / 10) +DD + LY)
      END

*ROUTINE G:  Convert from YYYY,MM,DD to day-of-week
*  Reference: a Zeller's congruence algorithm submitted by Mike Carroll
*  of Candle Corp to the bit.listserv.ibm-main group on May 5, 1995.
*    (I figured out a typo in his original posting, which I hope I didn't
*     repeat myself)

* Input: YYYY, MM, DD Output: day-of-week (0-6, 0=Sunday, 1=Monday, etc.)
* EX: Nov 1, 1996 results in DOW = 5

      SUBROUTINE cvt_date_g(yyyy,mm,dd,dow)
      IMPLICIT NONE
      INTEGER yyyy,mm,dd,dow
      INTEGER wmm,wyyyy
      IF (MM .LT. 3) THEN
        WMM   = MM + 12
        WYYYY = YYYY - 1
      ELSE
        WMM   = MM
        WYYYY = YYYY
      ENDIF
      DOW = MOD((DD + 1 + (MM * 2) + INT(((WMM + 1) * 3) / 5) +
     &  WYYYY + INT(WYYYY / 4) -
     &  INT(WYYYY / 100) + INT(WYYYY / 400)),7)



*These conversions were apparently needed often enough in the
*1970-s that several CACM algorithms were published to address
*them. To get the routines right is a non-trivial task, and the
*formulae are not always intuitively obvious (e.g., look up
*Zeller's Congruence and the Doomsday Rule).

*If you succeed in getting these (or equivalent) routines into
*Fortran 20xx, note that "around the year 4317 we will have to
*drop a day"!


      END
