pro retrieve_mpaiondata,satellitearr=satellitearr, $
	yeararr=yeararr, $
	montharr=montharr, $
	dayarr=dayarr, $
	cdfpath=cdfpath, $
	reretrieve=reretrieve, $
	dataformat=dataformat, $
	sav2bin=sav2bin 
; this pro is used to retrieve the data related with proton 3-D distribution function,
; which can be used for identifying ring distribution 
; 2009/10/22
; usage example
; .r readcdf.pro
; .r retrieve_mpaiondata.pro
; retrieve_mpaiondata,sat=['l7','l4','l0','a1','a2'],year=[2005],month=[8],day=[30 31],/reretr
; retrieve_mpaiondata,sat=['l7','l4','l0','a1','a2'],year=[2005],month=[9],day=indgen(7)+1,/reretr

; retrieve_mpaiondata,sat=['l7','01'],year=[2001],month=[4,5],day=indgen(31)+1
; will retrieve data from satellite l7 and 01 during the peroid 2001 April and May
; test cdf file : l7_h0_mpa_20010405_v02.cdf 
; retrieve_mpaiondata,sat=['l7'],year=[2001],month=[4],day=[5],/reretr,cdfpath=['']


; 2009/10/22
; save variable from the cdf format as .sav format
;    Pcounts, etimesg, effecp, ecp, theta_B, phi_B, Ae, $
;    theta_LK,phi_LK,xyzgeo, uthours

; 2009/10/23
; added /reretrieve option
; added addtional check on cdf file size, less than 1k byte to be considered as empty file    

; 2009/10/24
; added dataformat option, '.sav', '.bin'

; 2009/10/26
; added the thermal plasma density : dens_lop, dens_hip, dens_e, flags, Poten

; 2009/10/27
; added sav2bin option :  conversion from .sav to .bin






;filename=dialog_pickfile(/READ,Filter='/n/projects/space/disks/data4/mpa/cCDF/*.cdf')
;filename='l7_h0_mpa_20010405_v02.cdf'


if (N_elements(cdfpath) eq 0) then begin
  cdfpath=''
;  cdfpath='/n/projects/space/disks/data4/mpa/cCDF/'
endif
if (N_elements(satellitearr) eq 0) then satellitearr=['l0','l1','l4','l7','a2','a1']
if (N_elements(yeararr) eq 0) then yeararr=[2001]
if (N_elements(montharr) eq 0) then montharr=indgen(12)+1
if (N_elements(dayarr) eq 0) then dayarr=indgen(31)+1
if (N_elements(reretrieve) eq 0) then reretrieve=0
if (N_elements(dataformat) eq 0) then dataformat='.bin'
if (N_elements(sav2bin) eq 0) then sav2bin=0
dataformat=strlowcase(dataformat)
print,dataformat
sizelimit=1000;

for isat=0,n_elements(satellitearr)-1 do begin
for iyear=0,n_elements(yeararr)-1 do begin
for imonth=0,n_elements(montharr)-1 do begin
for iday=0,n_elements(dayarr)-1 do begin

satellite=satellitearr[isat]
dates=[yeararr[iyear],montharr[imonth],dayarr[iday]]
datestr=string(dates,format='(i4.4,i2.2,i2.2)')

;filename='l7_h0_mpa_20010405_v02.cdf'
filename=strjoin([satellite,'_h0_mpa_',datestr,'_v02.cdf'])
cdffile=strjoin([cdfpath,filename])
savfile=strjoin([filename,dataformat])
icdffile=FILE_TEST(cdffile)
isavfile=FILE_TEST(savfile)

if ((icdffile eq 1) and ((isavfile eq 0) or (reretrieve eq 1) )) then begin
	print,cdffile,' found: processing'
	cdffileinfo=FILE_INFO(cdffile)
	cdffilesz=cdffileinfo.size
	
;****************
	if (cdffilesz le sizelimit) then begin
	print,cdffile,' : empty data, ',cdffilesz, ' bytes: skipped'

	endif else begin
;*****

	data=readcdf(cdffile)
	Pcounts=data.Pcounts.data;
	sz=size(Pcounts);
	nEn=40;
	nD=6;
	nL=24;
	nrec=sz[1];

	etimesg=data.etimesg.data; geometric fac
	effecp=data.effecp.data; energy-dependent efficiencies
	ecp=data.ecp.data; center energy of 40 channels in ev

	; Inferred field direction
	; thetaphi_e ;  [nrec,2]
	theta_B=data.thetaphi_e.data[*,0]; [nrec]
	phi_B=data.thetaphi_e.data[*,1]; [nrec]
	Ae=data.temp_e.data[*,0]/data.temp_e.data[*,1];

	; Look direction
	rad=!pi/180
	theta_LK=(data.theta_l.data+data.theta_u.data)*0.5*rad; [nD] in rad
	phi_LK=data.azanglp.data*rad; [nrec,nEn,nL] in rad
	xyzgeo=data.xyzgeo.data
	uthours=data.uthours.data
	dens_lop=data.dens_lop.data
	dens_hip=data.dens_hip.data
	dens_e=data.dens_e.data
	flags=data.flags.data
	Poten=data.Poten.data

  if strcmp(dataformat,'.sav',4,/fold_case) then begin
	  save, Pcounts, etimesg, effecp, ecp, theta_B, phi_B, Ae,  $
		theta_LK,phi_LK,xyzgeo, uthours, dens_hip, dens_lop, dens_e,flags, Poten, $
		filename=savfile
		print,savfile,' saved, successed '
  endif else if strcmp(dataformat,'.bin',4,/fold_case) then begin
    openw, lun, savfile, /get_lun, /F77_UNFORMATTED
    sz=size(Pcounts)
    print,sz
    writeu,lun,sz[1:4]
    writeu,lun, Pcounts, etimesg, effecp, ecp, theta_B, phi_B, Ae, $
    theta_LK,phi_LK,xyzgeo, uthours, dens_hip,dens_lop,dens_e, flags, Poten
    free_lun,lun   
    print,savfile,' saved, successed '
  endif else begin
  endelse






  endelse 
;****************
endif else begin
	print,isavfile
	print,savfile
	if(isavfile eq 1) then begin
;	savfile=strjoin([filename,dataformat])
;	icdffile=FILE_TEST(cdffile)
		print,savfile,' exists: successed'
    		restore,savfile
		binfile=strjoin([filename,'.bin'])
    		openw, lun, binfile, /get_lun, /F77_UNFORMATTED
    		sz=size(Pcounts)
    		print,sz
    		writeu,lun,sz[1:4]
    		writeu,lun, Pcounts, etimesg, effecp, ecp, theta_B, phi_B, Ae, $
    		theta_LK,phi_LK,xyzgeo, uthours, dens_hip, dens_lop, dens_e, flags, Poten
    		free_lun,lun   
    		print,savfile,' saved, successed '
    		print,binfile,' saved, successed '
	endif else if (icdffile eq 0) then begin
		print,cdffile,' not exist: skipped'
	endif else begin
		print,cdffile,' error: skipped'
	endelse
endelse
endfor; iday
endfor; imonth
endfor; iyear
endfor; isat
end

